'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _ListBase2 = require('./supports/ListBase');

var _ListBase3 = _interopRequireDefault(_ListBase2);

var _ListItemRender = require('./ListItemRender');

var _ListItemRender2 = _interopRequireDefault(_ListItemRender);

var _ListSkin = require('../skins/ListSkin');

var _ListSkin2 = _interopRequireDefault(_ListSkin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 列表组件，依次排列显示多个子项。可设置是否允许选中。
 * 
 * <h2>皮肤结构</h2>
 * <ul>
 *   <li>container 表示子项的父容器</li>
 * </ul>
 * 
 * 默认子项为{@link ListItemRender}，List会给子项传下面几项数据
 * data——携带的数据
 * index——当前数据在数据集合的位置
 * selected——当前项是否选中
 * label——默认的文本内容
 * 
 * 支持自定义子项，如需自定义子项，设置itemClassType的值即可。自定义子项建议继承自ListItemRender
 * @example <caption>创建List</caption>
 * <List data={this.state.articleData} itemClassType={ArticleItem} style={{ width: '690px', marginTop: '30px' }} />
 * 
 * //ArticleItem类代码如下
 * class ArticleItem extends ListItemRender {
 * 	render() {
 * 		let article = this.data;
 * 		
 * 		return (
 * 			<div className='articleItem'>
 * 				<div className='subject'>来自话题：{article.subject}</div>
 * 				<div className='title'>{article.title}</div>
 * 				<div className='author'>{article.author}</div>
 * 				<div className='content'>{article.content}</div>
 * 			</div>
 * 		);
 * 	}
 * }
 * export default ArticleItem;
 * 
 * //创建文章数据代码如下
 * let data = [];
 * 	for (let i = 0; i < 10; i++) {
 * 		let article = new ArticleData('JAVASCRIPT', '标题', 'Skillness', '文章内容', 5, 100, 8);
 * 		data.push(article);
 * 	}
 * 	this.state = { articleData: data };
 * 
 * @author xujunjie
 * @mail yinhunfeixue@163.com
 * @date 2017-10-16 09:27:22
 * 
 * @extends ListBase
 */
var List = function (_ListBase) {
	_inherits(List, _ListBase);

	function List() {
		var _ref;

		var _temp, _this, _ret;

		_classCallCheck(this, List);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = List.__proto__ || Object.getPrototypeOf(List)).call.apply(_ref, [this].concat(args))), _this), _this.item_changeHandler = function (event) {
			var target = event.currentTarget;
			if (target.selected) {
				var index = target.index;
				_this.selectedIndex = index;
			}
		}, _temp), _possibleConstructorReturn(_this, _ret);
	}

	_createClass(List, [{
		key: '_createItemProps',
		value: function _createItemProps(data, index) {
			var result = _get(List.prototype.__proto__ || Object.getPrototypeOf(List.prototype), '_createItemProps', this).call(this, data, index);
			if (this.selectedAble && index === this.selectedIndex) {
				result.selected = true;
			} else {
				result.selected = false;
			}
			result.labelFunction = this.labelFunction;
			result.selectedAble = this.selectedAble;
			result.changeHandler = this.item_changeHandler;
			return result;
		}
	}, {
		key: '_getDefaultSKinClass',
		value: function _getDefaultSKinClass() {
			return _ListSkin2.default;
		}

		/**
   * @inheritdoc
   */

	}, {
		key: '_initProperty',
		value: function _initProperty() {
			_get(List.prototype.__proto__ || Object.getPrototypeOf(List.prototype), '_initProperty', this).call(this);
			this._selectedAble = true;
			this._itemClassType = _ListItemRender2.default;
		}

		/**
   * 是否可选中
   * 
   * @type {boolean}
   * @default true
   */

	}, {
		key: 'selectedAble',
		get: function get() {
			return this._selectedAble;
		},
		set: function set(value) {
			if (this._selectedAble !== value) {
				this._selectedAble = value;
				if (this._selectedAble === false) {
					this.selectedIndex = -1;
				}
				this.forceUpdate();
			}
		}
	}]);

	return List;
}(_ListBase3.default);

exports.default = List;
//# sourceMappingURL=List.js.map