'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _ObjectUtil = require('../utils/ObjectUtil');

var _ObjectUtil2 = _interopRequireDefault(_ObjectUtil);

var _UIComponent2 = require('./UIComponent');

var _UIComponent3 = _interopRequireDefault(_UIComponent2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


/**
 * SkinnableComponent 类定义可设置外观的组件的基类。SkinnableComponent 类所使用的外观通常是 SkinBase 类的子类
 * 继承此类定义组件时，通常需要重写以下方法
 * 
 * //设置组件属性的默认值  {@link SkinableComponent#_initProperty _initProperty}
 * _initProperty()    		
 * 
 * //设置组件默认皮肤 {@link SkinableComponent#_getDefaultSKinClass _getDefaultSKinClass}
 * _getDefaultSKinClass
 * 
 * //设置皮肤可能出现的状态 {@link SkinableComponent#_getCurrentSkinStatus _getCurrentSkinStatus}
 * _getCurrentSkinStatus()
 * 
 * //初始化皮肤  {@link SkinableComponent#_initSkin _initSkin}
 * _initSkin()
 * 
 * @date    2017-09-05 11:24:58
 * 
 * @extends UIComponent
 */
var SkinableComponent = function (_UIComponent) {
	_inherits(SkinableComponent, _UIComponent);

	function SkinableComponent() {
		_classCallCheck(this, SkinableComponent);

		return _possibleConstructorReturn(this, (SkinableComponent.__proto__ || Object.getPrototypeOf(SkinableComponent)).apply(this, arguments));
	}

	_createClass(SkinableComponent, [{
		key: '_initProperty',


		/**
   * 设置组件属性的默认值
   * @protected
   */
		value: function _initProperty() {
			_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), '_initProperty', this).call(this);
			this._currentStatus = null;
			this._skinClass = this._getDefaultSKinClass();
			this._skin = null;
			this._selectedAble = true;
		}

		/**
   * 初始化皮肤部件的设置，在安装新皮肤、组件第一次渲染、组件更新后会调用
   * 通常在这此函数处理皮肤部件的事件监听
   * @protected
   */

	}, {
		key: '_initSkin',
		value: function _initSkin() {}

		/**
   * 表示组件内容是否可选中
   * @type {boolean}
   * 
   * @default true
   */

	}, {
		key: 'findSkinPart',


		/**
   * 从皮肤中获取指定部件
   * 在皮肤中通过设置ref属性设置名称
   * @param {string} name 部件名称
   * 
   * @return {object} 部件实例
   */
		value: function findSkinPart(name) {
			if (this._skin != null) {
				return this.refs[name];
			}
		}
	}, {
		key: 'componentWillMount',
		value: function componentWillMount() {
			this._installSkin();
			_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'componentWillMount', this).call(this);
		}
	}, {
		key: 'componentDidMount',
		value: function componentDidMount() {
			var _this2 = this;

			if (this.rootComponent) {
				this.rootComponent.onselectstart = function () {
					return _this2._selectedAble;
				};
				this.rootComponent.addEventListener('click', this.clickHandler.bind(this), true);
			}

			this._initSkin();
			_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'componentDidMount', this).call(this);
		}
	}, {
		key: 'componentDidUpdate',
		value: function componentDidUpdate(prevProps, prevState) {
			this._initSkin();
			_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'componentDidUpdate', this).call(this, prevProps, prevState);
		}

		/**
   * 单击事件处理函数
   * @protected
   */

	}, {
		key: 'clickHandler',
		value: function clickHandler(event) {
			if (!this.enable) {
				event.stopPropagation();
			}
		}

		/**
   * @private
   * @param {object} nextProps 
   */

	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			if (_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'componentWillReceiveProps', this)) {
				_get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'componentWillReceiveProps', this).call(this, nextProps);
			}

			//把props的值传递到rootComponent中，例如设置了样式和事件，实际上会被应用到rootComponent中
			_ObjectUtil2.default.Fill(this.rootComponent, _ObjectUtil2.default.filterDifferentValue(this.props, this._getEffectProp()), false, true);
		}
		/**
   * 用于此组件的皮肤类的名称，皮肤类需要实现render方法，建议继承自{@link SkinBase}
   * @type {Class}
   */

	}, {
		key: '_getCurrentSkinStatus',


		/**
   * 获取当前皮肤状态，在子类中重写此方法，以设置不同的皮肤状态
   * @protected
   * 
   * @property {string} unable 组件不可用状态
   * 
   * @return {string} 当前皮肤状态
   */
		value: function _getCurrentSkinStatus() {
			if (this.enable) {
				return '';
			} else {
				return 'unable';
			}
		}

		/**
   * 默认皮肤类，当继承此类自定义组件时，一般需要重写此方法
   * 
   * @protected
   * @type {Class}
   * 
   * @return {Class} 皮肤类
   */

	}, {
		key: '_getDefaultSKinClass',
		value: function _getDefaultSKinClass() {
			return null;
		}

		/**
   * 组件的根显示对象
   * @type {div}
   * @readonly
   */

	}, {
		key: '_installSkin',


		/**
   * 安装皮肤，组件初始化或者改变skinClass后，会执行此方法安装新皮肤
   * @private
   */
		value: function _installSkin() {
			if (this._skinClass != null) {
				this._skin = new this._skinClass(this);
				this._currentStatus = null;
				this._updateSkinStatus();
				if (this.inited) {
					this._initSkin();
				}
			}
		}
	}, {
		key: '_getDefaultStyle',
		value: function _getDefaultStyle() {
			if (this._skin) {
				return this._skin.rootStyle;
			}
		}

		/**
   * @private
   */

	}, {
		key: '_unstallSkin',
		value: function _unstallSkin() {
			if (this._skin != null) {
				if (this._skin.dispose) {
					this._skin.dispose();
				}
				this._skin = null;
				this._currentStatus = null;
			}
		}

		/**
   * 更新皮肤状态
   * 
   * @private
   */

	}, {
		key: '_updateSkinStatus',
		value: function _updateSkinStatus() {
			if (this._skin != null) {
				var status = this._getCurrentSkinStatus();
				if (status !== this._currentStatus) {
					this._skin.styleChange(status, this._currentStatus);
					this._currentStatus = status;
					this.forceUpdate();
				}
			}
		}

		/**
   * @private
   */

	}, {
		key: '_getEffectProp',
		value: function _getEffectProp() {
			var obj = _ObjectUtil2.default.clone(this.props);
			delete obj.style;
			return obj;
		}
	}, {
		key: '_getSkinClass',
		value: function _getSkinClass() {
			var result = [];
			if (this._skin && this._skin.rootClass) {
				result.push(this._skin.rootClass);
			}
			if (this.props.className) {
				result.push(this.props.className);
			}

			return result.join(' ');
		}
	}, {
		key: 'render',
		value: function render() {
			var result = _react2.default.createElement(
				'div',
				_extends({ ref: 'rootComponent', 'remote-root': 'true'
				}, _ObjectUtil2.default.Fill(document.createElement('div'), this._getEffectProp(), false, true), {
					style: _ObjectUtil2.default.Merge({ display: 'inline-block' }, this.props.style, this._getDefaultStyle()), className: this._getSkinClass() }),
				this._skin ? this._skin.render() : _react2.default.createElement('span', null)
			);

			return result;
		}
	}, {
		key: 'selectedAble',
		get: function get() {
			return this._selectedAble;
		},
		set: function set(value) {
			this._selectedAble = value;
		}
	}, {
		key: 'skinClass',
		get: function get() {
			return this._skinClass;
		},
		set: function set(value) {
			if (this._skinClass !== value) {
				this._unstallSkin();
				this._skinClass = value;
				this._installSkin();
			}
		}

		/**
   * @inheritdoc
   */

	}, {
		key: 'enable',
		get: function get() {
			return _get(SkinableComponent.prototype.__proto__ || Object.getPrototypeOf(SkinableComponent.prototype), 'enable', this);
		},
		set: function set(value) {
			if (this._enable !== value) {
				this._enable = value;
				this._updateSkinStatus();
			}
		}
	}, {
		key: 'rootComponent',
		get: function get() {
			return this.refs.rootComponent;
		}
	}, {
		key: 'style',
		get: function get() {
			if (this.rootComponent) {
				return this.rootComponent.style;
			}
			return null;
		}
	}]);

	return SkinableComponent;
}(_UIComponent3.default);

exports.default = SkinableComponent;
//# sourceMappingURL=SkinableComponent.js.map