'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _set = function set(object, property, value, receiver) { var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent !== null) { set(parent, property, value, receiver); } } else if ("value" in desc && desc.writable) { desc.value = value; } else { var setter = desc.set; if (setter !== undefined) { setter.call(receiver, value); } } return value; };

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _SkinableComponent2 = require('./SkinableComponent');

var _SkinableComponent3 = _interopRequireDefault(_SkinableComponent2);

var _TextInputSkin = require('../skins/TextInputSkin');

var _TextInputSkin2 = _interopRequireDefault(_TextInputSkin);

var _TextEvent = require('../events/TextEvent');

var _TextEvent2 = _interopRequireDefault(_TextEvent);

var _RemoteEvent = require('../events/RemoteEvent');

var _RemoteEvent2 = _interopRequireDefault(_RemoteEvent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 单行文本输入框
 * 可设置允许输入的字符，可设置错误提示
 * <h2>皮肤结构</h2>
 * <ul>
 *   <li>inputNode 文本输入框，类型必须是 &lt;input type='text'></li>
 * </ul>
 * @example <caption>创建TextInput</caption>
 * //创建一个输入提示：手机号，只允许输入数字的输入框
 * <TextInput ref='tbPhone' restrict='0-9' placeholder='手机号' errorText={this.state.phoneError} />
 * 
 * @example <caption>创建文本框，并获取值</caption>
 * <TextInput ref='tbPhone' restrict='0-9' placeholder='手机号' errorText={this.state.phoneError} />
 * <Button label='获取输入的值' onClick={this.btnClickHandler} />
 * 
 * btnClickHandler = (event) => {
 *   //通过refs获取文本,也可以在onChange事件中获取event.currentTarget.text获取值，并保存到this.state中
 *	 let phone = this.refs.tbPhone.text;
 *	 et phoneError = (phone.length > 0 ? '' : '请输入手机号');
 *	 this.setState({phoneError: phoneError});
 *	 if (phoneError.length === 0) {
 *		 console.log(phone);
 *	 }
 * }
 * 
 * @author : xujunjie 
 * @mail   : yinhunfeixue@163.com
 * @date   : 2017-9-12 17:08:29
 * 
 * @extends SkinableComponent
 */
var TextInput = function (_SkinableComponent) {
	_inherits(TextInput, _SkinableComponent);

	function TextInput() {
		var _ref;

		var _temp, _this, _ret;

		_classCallCheck(this, TextInput);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = TextInput.__proto__ || Object.getPrototypeOf(TextInput)).call.apply(_ref, [this].concat(args))), _this), _this._inputNodeBlurHandler = function (event) {
			_this.dispatchEvent(new _RemoteEvent2.default(_RemoteEvent2.default.BLUR));
		}, _this._inputNodeFocusHandler = function (event) {
			_this.dispatchEvent(new _RemoteEvent2.default(_RemoteEvent2.default.FOCUS));
		}, _this._inputNodeKeyDownHandler = function (event) {
			if (event.keyCode === 13) {
				_this.dispatchEvent(new _RemoteEvent2.default(_RemoteEvent2.default.ENTER));
			}
		}, _this.inputNodeChangeHandler = function (event) {
			//如果是组合输入，且内容只有英文字母，表示未输入完成，不处理
			//（因为IOS自带输入法在输入汉字时，会把先输入的字母也放到输入框中
			if (event.inputType === 'insertCompositionText' && event.data && !/[\u4e00-\u9fa5]/.test(event.data)) {
				return;
			} else {
				_this.text = _this.inputNode.value;
			}
		}, _temp), _possibleConstructorReturn(_this, _ret);
	}

	_createClass(TextInput, [{
		key: '_initProperty',


		/**
   * @inheritdoc
   */
		value: function _initProperty() {
			_get(TextInput.prototype.__proto__ || Object.getPrototypeOf(TextInput.prototype), '_initProperty', this).call(this);
			this._restrict = null;
			this._maxChars = -1;
			this._editable = true;
			this._displayAsPassword = false;
			this._text = '';
			this._errorText = '';
			this._changeHandler = null;
			this._enterHandler = null;
			this._blurHandler = null;
			this._focusHandler = null;
			this._placeholder = '';
		}

		/**
   * 
   * @type {}
   * @default 
   */

	}, {
		key: '_getDefaultSKinClass',


		/**
   * @inheritdoc
   */
		value: function _getDefaultSKinClass() {
			return _TextInputSkin2.default;
		}

		/**
   * @private
   */

	}, {
		key: '_adaptText',
		value: function _adaptText() {
			var newText = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

			//如果未设置newText，则使用当前值进行计算
			if (newText === null) {
				newText = this._text;
			}

			if (newText != null && newText.length > 0) {
				//检查正则匹配，删除不匹配的字符
				if (this._restrict != null) {
					var regString = '[' + (this._restrict[0] === '^' ? this._restrict.substring(1) : '^' + this._restrict) + ']';
					var reg = new RegExp(regString, 'g');
					newText = newText.replace(reg, '');
				}

				//检查长度
				if (this._maxChars >= 0 && newText.length > this._maxChars) {
					newText = newText.substring(0, this._maxChars);
				}
			}

			if (this._text !== newText) {
				var oldText = this._text;
				this._text = newText;
				this.forceUpdate();
				this.dispatchEvent(new _TextEvent2.default(_TextEvent2.default.TEXT_CHANGE, this._text, oldText));
			}

			if (this.inputNode != null) {
				this.inputNode.value = this._text;
			}
		}

		/**
   * 文本变化事件的处理函数，事件类型:{@link TextEvent#TEXT_CHANGE}
   * @default null;
   * @type {function}
   */

	}, {
		key: 'focus',


		/**
   * 强制获得焦点
   */
		value: function focus() {
			if (this.inputNode) {
				this.inputNode.focus();
			}
		}

		/**
   * 强制失去焦点
   */

	}, {
		key: 'blur',
		value: function blur() {
			if (this.inputNode) {
				this.inputNode.blur();
			}
		}

		/**
   * 文本框获得焦点事件
   * @type {function}
   * @default null;
   */

	}, {
		key: '_initSkin',


		/**
   * @inheritdoc
   */
		value: function _initSkin() {
			if (this.inputNode != null) {
				this.inputNode.addEventListener('input', this.inputNodeChangeHandler);
				this.inputNode.addEventListener('keydown', this._inputNodeKeyDownHandler);
				this.inputNode.addEventListener('blur', this._inputNodeBlurHandler);
				this.inputNode.addEventListener('focus', this._inputNodeFocusHandler);
				this.inputNode.type = this._displayAsPassword ? 'password' : 'text';
				this.inputNode.readOnly = this._editable ? '' : 'true';
				this.inputNode.disabled = this._enable ? '' : 'true';
				this._adaptText();
			}
		}

		/**
   * @inheritdoc
   */

	}, {
		key: '_unstallSkin',
		value: function _unstallSkin() {
			if (this.inputNode != null) {
				this.inputNode.removeEventListener('input', this.inputNodeChangeHandler);
				this.inputNode.removeEventListener('keydown', this._inputNodeKeyDownHandler);
				this.inputNode.removeEventListener('blur', this._inputNodeBlurHandler);
				this.inputNode.removeEventListener('focus', this._inputNodeFocusHandler);
			}
			_get(TextInput.prototype.__proto__ || Object.getPrototypeOf(TextInput.prototype), '_unstallSkin', this).call(this);
		}
		/**
   * @private
   */


		/**
   * @private
   */


		/**
   * @private
   */


		/**
   * @private
   */

	}, {
		key: 'enable',
		get: function get() {
			return this._enable;
		},
		set: function set(value) {
			if (this.enable !== value) {
				if (this.inputNode) {
					this.inputNode.disabled = value ? '' : 'true';
				}
				_set(TextInput.prototype.__proto__ || Object.getPrototypeOf(TextInput.prototype), 'enable', value, this);
			}
		}

		/**
   * 用户填写输入字段的提示
   * @default ''
   * @memberof TextInput
   */

	}, {
		key: 'placeholder',
		get: function get() {
			return this._placeholder;
		},
		set: function set(value) {
			if (this._placeholder !== value) {
				this._placeholder = value;
				this.inputNode.placeholder = value;
			}
		}
	}, {
		key: 'changeHandler',
		get: function get() {
			return this._changeHandler;
		},
		set: function set(value) {
			if (this._changeHandler !== value) {
				this._changeEventListener(_TextEvent2.default.TEXT_CHANGE, value, this._changeHandler);
				this._changeHandler = value;
			}
		}

		/**
   * 输入回车键的处理函数，事件类型:{@link RemoteEvent#ENTER}
   * @default null
   */

	}, {
		key: 'enterHandler',
		get: function get() {
			return this._enterHandler;
		},
		set: function set(value) {
			if (this._enterHandler !== value) {
				this._changeEventListener(_RemoteEvent2.default.ENTER, value, this._enterHandler);
				this._enterHandler = value;
			}
		}
	}, {
		key: 'focusHandler',
		get: function get() {
			return this._focusHandler;
		},
		set: function set(value) {
			if (this._focusHandler !== value) {
				this._changeEventListener(_RemoteEvent2.default.FOCUS, value, this._focusHandler);
				this._focusHandler = value;
			}
		}

		/**
   * 文本框失去焦点事件
   * @listens RemoteEvent:BLUR
   * @type {function}
   * @default null;
   */

	}, {
		key: 'blurHandler',
		get: function get() {
			return this._blurHandler;
		},
		set: function set(value) {
			if (this._blurHandler !== value) {
				this._changeEventListener(_RemoteEvent2.default.BLUR, value, this._blurHandler);
				this._blurHandler = value;
			}
		}

		/**
   * 错误提示文本
   * 
   * @default ''
   * @type {string}
   */

	}, {
		key: 'errorText',
		get: function get() {
			return this._errorText;
		},
		set: function set(value) {
			if (this._errorText !== value) {
				this._errorText = value;
				this.forceUpdate();
			}
		}

		/**
   * HTML的输入框，在皮肤类中定义，ref='inputNode'
   */

	}, {
		key: 'inputNode',
		get: function get() {
			return this.findSkinPart('inputNode');
		}

		/**
   * 指示用户可输入到文本字段中的字符集
   * restrict 属性的值为一串字符，只能在文本字段中输入该字符串中的字符。从左向右扫描该字符串。可以使用连字符 (-) 指定一个范围。
   * 例如0-9，可输入数字， ^a-z表示可输入小写字母之外的所有字符
   * @type {string}
   */

	}, {
		key: 'restrict',
		get: function get() {
			return this._restrict;
		},
		set: function set(value) {
			if (value !== this._restrict) {
				this._restrict = value;
				this._adaptText();
			}
		}

		/**
   * 允许输入的最大字符数量,-1表示不限制
   * 
   * @default -1
   * @type {Number}
   */

	}, {
		key: 'maxChars',
		get: function get() {
			return this._maxChars;
		},
		set: function set(value) {
			if (value !== this._maxChars) {
				this._maxChars = value;
				this._adaptText();
			}
		}

		/**
   * 是否可编辑
   * 
   * @default true
   * @type {boolean}
   */

	}, {
		key: 'editable',
		get: function get() {
			return this._editable;
		},
		set: function set(value) {
			if (value !== this._editable) {
				this._editable = value;
				if (this.inputNode != null) {
					this.inputNode.readOnly = this._editable ? '' : 'true';
				}
			}
		}

		/**
   * 设置或者输入的文本内容。
   * 此属性，会受到maxChar、restrict的影响。例如，设置maxChar=5，text='123456'，则最终的结果是text='12345'
   * @default ''
   * @type {string}
   */

	}, {
		key: 'text',
		get: function get() {
			return this._text;
		},
		set: function set(value) {
			if (value === null) {
				value = "";
			}
			this._adaptText(value);
		}

		/**
   * 是否显示为密码
   * @default false
   * @type {boolean}
   */

	}, {
		key: 'displayAsPassword',
		get: function get() {
			return this._displayAsPassword;
		},
		set: function set(value) {
			if (value !== this._displayAsPassword) {
				this._displayAsPassword = value;
				if (this.inputNode != null) {
					this.inputNode.type = this._displayAsPassword ? 'password' : 'text';
				}
			}
		}
	}]);

	return TextInput;
}(_SkinableComponent3.default);

exports.default = TextInput;
//# sourceMappingURL=TextInput.js.map