'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _react = require('react');

var _ObjectUtil = require('../utils/ObjectUtil');

var _ObjectUtil2 = _interopRequireDefault(_ObjectUtil);

var _EventDispatcher = require('../core/EventDispatcher');

var _EventDispatcher2 = _interopRequireDefault(_EventDispatcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * UIComponent 类是所有可视组件（交互式和非交互式）的基类
 * 
 * <b>事件</b>
 * 组件在某些情况下，例如点击时，会抛出事件，可通过addEventListener监听事件。
 * 为了方便使用，有些常用事件还会提供一个快捷属性，例如onClick属性
 * 示例参考 {@link UIComponent#addEventListener addEventListener}
 * 
 * <b>forceUpdate</b>
 * 我们重写了forceUpdate函数，确保在componentWillMount之后执行，防止报错
 */
var UIComponent = function (_Component) {
	_inherits(UIComponent, _Component);

	function UIComponent(props) {
		_classCallCheck(this, UIComponent);

		var _this = _possibleConstructorReturn(this, (UIComponent.__proto__ || Object.getPrototypeOf(UIComponent)).call(this, props));

		_this._listener = new _EventDispatcher2.default();

		//先初始化所有属性
		_this._initProperty();

		//如果在props中设置了属性，则使用props中的属性替换默认值
		_ObjectUtil2.default.Fill(_this, props);
		return _this;
	}

	/**
   * 添加事件监听。如果相同类型的事件，已添加过相同的处理函数，则不再添加
  * 为了方便使用，有些常用事件还会提供一个快捷属性，例如onClick属性
   * @param {string} type 事件类型
   * @param {function} fun 要添加的函数，函数有一个参数event 
   * @param {number} priority 优先级，值越大，越先执行。同一优先级的，按添加顺序执行，先添加先执行
  * 
  * @example
 	 * let button = new Button();
   * 
   * //下面的两句执行后，都会给click事件绑定btnClickHandler的方法
   * button.onClick = btnClickHandler;
   * //button.addEventListener('click', btnClickHandler);
   * 
   * //第二次执行时
   * //前者，会先移除btnClickHandler，再绑定到btnClickHandler2，当click事件发生时，只触发btnClickHandler
   * //后者会添加btnClickHandler2，当click事件发生时，会同时触发btnClickHandler和btnClickHandler2
   * button.onClick = btnClickHandler2;
   * //button.addEventListener('click', btnClickHandler2);
  * 
  * function btnClickHandler(event){
  * 	//在此处对事件进行处理
  * }
   */


	_createClass(UIComponent, [{
		key: 'addEventListener',
		value: function addEventListener(type, fun, priority) {
			this._listener.addEventListener(type, fun, priority);
		}

		/**
  * 移除事件监听
  * @param {string} type 事件类型
  * @param {function} fun 函数
  */

	}, {
		key: 'removeEventListener',
		value: function removeEventListener(type, fun) {
			this._listener.removeEventListener(type, fun);
		}

		/**
  * 判断是否注册了针对某个事件类型的监听
  * @param {string} type 事件类型
  */

	}, {
		key: 'hasEventListener',
		value: function hasEventListener(type) {
			return this._listener.hasEventListener(type);
		}

		/**
  * 将事件分派到事件流中
  * @param {RemoteEvent} event 要发出的事件
  */

	}, {
		key: 'dispatchEvent',
		value: function dispatchEvent(event) {
			var e = event.clone();
			e.target = event.target === null ? this : event.target;
			e.currentTarget = this;
			this._listener.dispatchEvent(e);
		}

		/**
   * 切换监听函数，对于事件eventType，此方法先移除oldFunction，再添加newFunction
   * @param {string} eventType 事件类型
   * @param {function} newFunction 新的事件处理函数
   * @param {function} oldFunction 旧的事件处理函数
   * 
   * @protected
   */

	}, {
		key: '_changeEventListener',
		value: function _changeEventListener(eventType, newFunction, oldFunction) {
			if (oldFunction != null) {
				this.removeEventListener(eventType, oldFunction);
			}
			if (newFunction != null) {
				this.addEventListener(eventType, newFunction);
			}
		}

		/**
   * 设置组件属性的默认值
   * @protected
   */

	}, {
		key: '_initProperty',
		value: function _initProperty() {
			this._enable = true;
			this.inited = false;
		}

		/**
   * @private
   */

	}, {
		key: 'componentWillMount',
		value: function componentWillMount() {
			if (_get(UIComponent.prototype.__proto__ || Object.getPrototypeOf(UIComponent.prototype), 'componentWillMount', this)) {
				_get(UIComponent.prototype.__proto__ || Object.getPrototypeOf(UIComponent.prototype), 'componentWillMount', this).call(this);
			}
			this.inited = true;
		}

		/**
   * @private
   * @param {object} nextProps 
   */

	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			if (_get(UIComponent.prototype.__proto__ || Object.getPrototypeOf(UIComponent.prototype), 'componentWillReceiveProps', this)) {
				_get(UIComponent.prototype.__proto__ || Object.getPrototypeOf(UIComponent.prototype), 'componentWillReceiveProps', this).call(this, nextProps);
			}
			//把nextProps的值传到this
			_ObjectUtil2.default.Fill(this, _ObjectUtil2.default.filterDifferentValue(this, nextProps));
		}

		/**
   * 组件是否可以接受用户交互
   * @type {boolean}
   * 
   * @default true
   */

	}, {
		key: 'forceUpdate',


		/**
   * 重写了forceUpdate，只有willMount后，才会被触发。
   * react默认的forceUpdate未判断，但是在willMount前触发，会有警告
   */
		value: function forceUpdate() {
			if (this.inited) {
				_get(UIComponent.prototype.__proto__ || Object.getPrototypeOf(UIComponent.prototype), 'forceUpdate', this).call(this);
			}
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.inited = false;
		}
	}, {
		key: 'componentDidMount',
		value: function componentDidMount() {}
	}, {
		key: 'componentDidUpdate',
		value: function componentDidUpdate(prevProps, prevState) {}
	}, {
		key: 'enable',
		get: function get() {
			return this._enable;
		},
		set: function set(value) {
			if (this._enable !== value) {
				this._enable = value;
			}
		}
	}]);

	return UIComponent;
}(_react.Component);

exports.default = UIComponent;
//# sourceMappingURL=UIComponent.js.map