'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _EventDispatcher2 = require('./EventDispatcher');

var _EventDispatcher3 = _interopRequireDefault(_EventDispatcher2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 表示一个集合
 * @author xujunjie
 * @mail yinhunfeixue@163.com
 * @date 2017-11-27 17:43:36
 */
var Collection = function (_EventDispatcher) {
	_inherits(Collection, _EventDispatcher);

	/**
  * 实例化一个集合
  * @param {Array} data 数据源
  */
	function Collection() {
		var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

		_classCallCheck(this, Collection);

		var _this = _possibleConstructorReturn(this, (Collection.__proto__ || Object.getPrototypeOf(Collection)).call(this));

		_this._data = data ? data : [];
		return _this;
	}

	/**
  * 添加一项，如果此项在集合中已存在，则不添加
  * @param {*} item 要添加的项
  */


	_createClass(Collection, [{
		key: 'addItem',
		value: function addItem(item) {
			this.addItemAt(item, this._data.length);
		}

		/**
   * 在指定位置添加指定项。如果此项在集合中已存在，或者指定位置超出集合范围，不进行任何操作
   * @param {*} item 要添加的项
   * @param {Number} index 要插入的位置
   */

	}, {
		key: 'addItemAt',
		value: function addItemAt(item, index) {
			if (!this.hasItem(item) && index >= 0 && index <= this._data.length) {
				this._data.splice(index, 0, item);
			}
		}

		/**
   * 判断集合是否包含指定项
   * @param {*} item 要查找的项
   * 
   * @return {boolean} 集合是否包含指定项
   */

	}, {
		key: 'hasItem',
		value: function hasItem(item) {
			return this.getItemIndex(item) >= 0;
		}

		/**
   * 获取指定项在集合中的位置
   * @param {*} item 要查找的项
   * 
   * @return {Number} 指定项在集合中的位置，如果不存在返回-1
   */

	}, {
		key: 'getItemIndex',
		value: function getItemIndex(item) {
			return this._data.indexOf(item);
		}

		/**
   * 获取集合中指定位置的项，如果位置超出集合实际的范围，返回Null
   * @param {Number} index 位置
   * 
   * @return {*} 查找出的项
   */

	}, {
		key: 'getItemAt',
		value: function getItemAt(index) {
			if (index >= 0 && index < this._data.length) {
				return this._data[index];
			}
			return null;
		}

		/**
   * 从集合中移除指定项
   * @param {*} item 要移除的项
   */

	}, {
		key: 'removeItem',
		value: function removeItem(item) {
			var index = this.getItemIndex(item);
			if (index !== -1) {
				this._data.splice(index, 1);
			}
		}

		/**
   * 删除指定位置的项，如果位置超出集合实际范围，不进行任何操作
   * @param {Number} index 指定位置
   * 
   * @return {*} 被删除的项，如果未删除任何项，返回Null
   */

	}, {
		key: 'removeItemAt',
		value: function removeItemAt(index) {
			if (index >= 0 && index < this._data.length) {
				return this._data.splice(index, 1)[0];
			}
			return null;
		}

		/**
   * 清空集合
   */

	}, {
		key: 'clear',
		value: function clear() {
			this._data = [];
		}
	}]);

	return Collection;
}(_EventDispatcher3.default);

exports.default = Collection;
//# sourceMappingURL=Collection.js.map