'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _HtmlUtil = require('../utils/HtmlUtil');

var _HtmlUtil2 = _interopRequireDefault(_HtmlUtil);

var _Point = require('./Point');

var _Point2 = _interopRequireDefault(_Point);

var _EventDispatcher2 = require('../core/EventDispatcher');

var _EventDispatcher3 = _interopRequireDefault(_EventDispatcher2);

var _MoveEvent = require('../events/MoveEvent');

var _MoveEvent2 = _interopRequireDefault(_MoveEvent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 拖动控制类，通过指定鼠标响应对象、被拖动对象，可让鼠标在鼠标响应对象上移动时，被拖动对象同步移动
 * <b>请注意，拖动控制的是left、top的值，因此，只能left、top生效的布局模式（例如fixed, position），拖动功能才有效</b>
 * 
 * 可选是否允许拖到浏览器窗口外面
 * 可设置允许拖动的范围
 * 
 * @author : xujunjie 
 * @mail   : yinhunfeixue@163.com
 * @date   : 2017-9-25 15:21:13
 */
var Drager = function (_EventDispatcher) {
	_inherits(Drager, _EventDispatcher);

	function Drager() {
		_classCallCheck(this, Drager);

		var _this = _possibleConstructorReturn(this, (Drager.__proto__ || Object.getPrototypeOf(Drager)).call(this));

		_this.documentResizeHandler = function (event) {
			var position = _HtmlUtil2.default.getLeftTop(_this.displayTarget);
			_this.adptePosition(position.x, position.y);
		};

		_this.targetMouseDownHandler = function (event) {
			if (!_this.enable) {
				return;
			}
			_this._startMousePoint = _HtmlUtil2.default.getMouseEventPosition(event);
			if (_this._startMousePoint != null) {
				_this._startTargetPosition = _HtmlUtil2.default.getLeftTop(_this.displayTarget);
				document.addEventListener('mousemove', _this.documentMouseMoveHandler);
				document.addEventListener('mouseup', _this.documentMouseUpHandler);

				document.addEventListener('touchmove', _this.documentMouseMoveHandler);
				document.addEventListener('touchend', _this.documentMouseUpHandler);
				_this._draging = true;
				if (_this.hasEventListener(_MoveEvent2.default.MOVE_START)) {
					_this.dispatchEvent(new _MoveEvent2.default(_MoveEvent2.default.MOVE_START, _this._startTargetPosition.x, _this._startTargetPosition.y));
				}
			}
		};

		_this.documentMouseMoveHandler = function (event) {
			if (!_this.enable) {
				return;
			}
			var point = _this._updatepositonByEvent(event);
			if (point && _this.hasEventListener(_MoveEvent2.default.MOVEING)) {
				_this.dispatchEvent(new _MoveEvent2.default(_MoveEvent2.default.MOVEING, point.x, point.y));
			}
		};

		_this.documentMouseUpHandler = function (event) {
			if (!_this.enable) {
				return;
			}
			_this._draging = false;
			var point = _this._updatepositonByEvent(event);
			if (point && _this.hasEventListener(_MoveEvent2.default.MOVE_END)) {
				_this.dispatchEvent(new _MoveEvent2.default(_MoveEvent2.default.MOVE_END, point.x, point.y));
			}
			document.removeEventListener('mousemove', _this.documentMouseMoveHandler);
			document.removeEventListener('mouseup', _this.documentMouseUpHandler);

			document.removeEventListener('touchmove', _this.documentMouseMoveHandler);
			document.removeEventListener('touchend', _this.documentMouseUpHandler);
		};

		_this._enable = true;
		_this._draging = false;
		_this._minLeft = NaN;
		_this._maxLeft = NaN;
		_this._minTop = NaN;
		_this._maxTop = NaN;
		return _this;
	}

	/**
  * 是否正在拖动
  * @type {boolean}
  * 
  * @readonly
  */


	_createClass(Drager, [{
		key: 'bind',


		/**
   * 给指定的显示对象绑定拖动功能。
   * 通常，响应鼠标事件的对象和被拖动的对对象相同；但也支持，A对象响应鼠标事件，被拖动的是B对象
   * @param {htmlElement} target 响应鼠标事件的对象
   * @param {htmlElement} displayTarget 被拖动位置的对象，默认为null,表示和target相同
   * @param {boolean} outAble 是否允许拖出窗口范围，默认为false,表示不允许拖到窗口外
   * @param {Number} minLeft 拖动时，水平方向最小值，默认为NaN表示不限制
   * @param {Number} maxLeft 拖动时，水平方向最大值，默认为NaN表示不限制
   * @param {Number} minTop 拖动时，竖直方向最小值，默认为NaN表示不限制
   * @param {Number} maxTop 拖动时，竖直方向最大值，默认为NaN表示不限制
   */
		value: function bind(target) {
			var displayTarget = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
			var outAble = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
			var minLeft = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : NaN;
			var maxLeft = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : NaN;
			var minTop = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : NaN;
			var maxTop = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : NaN;

			if (target === null) {
				throw new RangeError('taget 不能为null');
			}

			if (displayTarget === null) {
				displayTarget = target;
			}
			this.outAble = outAble;
			this._minLeft = minLeft;
			this._maxLeft = maxLeft;
			this._minTop = minTop;
			this._maxTop = maxTop;
			//如果新的绑定对象和已绑定的对象相同，则只改变被拖动的显示对象
			if (this.target === target) {
				this.displayTarget = displayTarget;
				return;
			}

			//先移除之前的绑定，再添加新的绑定
			if (this.target != null) {
				this.unbind();
			}

			this.target = target;
			this.displayTarget = displayTarget;
			if (this.target != null) {
				this.target.addEventListener('mousedown', this.targetMouseDownHandler);
				this.target.addEventListener('touchstart', this.targetMouseDownHandler);
				window.addEventListener('resize', this.documentResizeHandler);
			}
		}

		/**
   * 解除绑定
   */

	}, {
		key: 'unbind',
		value: function unbind() {
			this._startMousePoint = null;
			this._startTargetPosition = null;
			this.displayTarget = null;
			if (this.target != null) {
				this.target.removeEventListener('mousedown', this.targetMouseDownHandler);
				this.target.removeEventListener('touchstart', this.targetMouseDownHandler);
				this.target = null;
			}
			document.removeEventListener('mousemove', this.documentMouseMoveHandler);
			document.removeEventListener('mouseup', this.documentMouseUpHandler);

			document.removeEventListener('touchmove', this.documentMouseMoveHandler);
			document.removeEventListener('touchend', this.documentMouseUpHandler);

			window.removeEventListener('resize', this.documentResizeHandler);
		}

		/**
   * @private
   */


		/**
   * @private
   */


		/**
   * @private
   */

	}, {
		key: '_updatepositonByEvent',
		value: function _updatepositonByEvent(event) {
			var point = _HtmlUtil2.default.getMouseEventPosition(event);
			if (point != null) {
				var endPosition = point.subPoint(this._startMousePoint).addPoint(this._startTargetPosition);
				return this.adptePosition(endPosition.x, endPosition.y);
			}
			return null;
		}

		/**
   * @private
   */

	}, {
		key: 'adptePosition',


		/**
   * @private
   */
		value: function adptePosition(x, y) {
			if (this.displayTarget) {
				this.displayTarget.style.left = x + 'px';
				this.displayTarget.style.top = y + 'px';

				var realPosition = new _Point2.default(x, y);
				//如果不能超出屏幕范围，则判断差值，并设置left,top
				if (!this.outAble) {
					var rectangle = _HtmlUtil2.default.getRectangleToWindow(this.displayTarget);
					var windowRectangle = _HtmlUtil2.default.getWindowRectangle();
					var beyondX = rectangle.right - windowRectangle.width;
					var beyondY = rectangle.bottom - windowRectangle.height;

					if (beyondX > 0) {
						realPosition.x = realPosition.x - beyondX;
					}
					if (beyondY > 0) {
						realPosition.y = realPosition.y - beyondY;
					}
					if (rectangle.x < 0) {
						realPosition.x = realPosition.x + -rectangle.x;
					}
					if (rectangle.y < 0) {
						realPosition.y = realPosition.y + -rectangle.y;
					}
				}

				//如果设置了可拖动范围同，则限制在范围内
				if (this._minLeft != null && !isNaN(this._minLeft) && realPosition.x < this._minLeft) {
					realPosition.x = this._minLeft;
				}

				if (this._maxLeft != null && !isNaN(this._maxLeft) && realPosition.x > this._maxLeft) {
					realPosition.x = this._maxLeft;
				}

				if (this._minTop != null && !isNaN(this._minTop) && realPosition.y < this._minTop) {
					realPosition.y = this._minTop;
				}

				if (this._maxTop != null && !isNaN(this._maxTop) && realPosition.y > this._maxTop) {
					realPosition.y = this._maxTop;
				}

				if (realPosition.x !== x || realPosition.y !== y) {
					this.displayTarget.style.left = realPosition.x + 'px';
					this.displayTarget.style.top = realPosition.y + 'px';
				}
				return realPosition;
			}
		}
	}, {
		key: 'draging',
		get: function get() {
			return this._draging;
		}

		/**
   * 是否生效。把此属性设置为false，可暂停拖动
   * 
   * @default true
   */

	}, {
		key: 'enable',
		get: function get() {
			return this._enable;
		},
		set: function set(value) {
			if (this._enable !== value) {
				this._enable = value;
			}
		}
	}]);

	return Drager;
}(_EventDispatcher3.default);

exports.default = Drager;
//# sourceMappingURL=Drager.js.map