'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * EventDispatcher类是可分派事件的所有运行时类的基类
 */
var EventDispatcher = function () {
  function EventDispatcher() {
    _classCallCheck(this, EventDispatcher);

    //events是一个字典，键:事件类型，值：每一项是{优先级，函数}的数组。结构如下
    //events[type] = [{priority, fun}, {priority, fun}, {priority, fun}, ...]
    this.events = new Map();
  }

  /**
   * 添加事件监听
   * 如果相同类型的事件，已添加过相同的处理函数，则不再添加
   * @param {string} type 事件类型
   * @param {function} fun 要添加的函数，函数有一个参数event 
   * @param {number} priority 优先级，值越大，越先执行。同一优先级的，按添加顺序执行，先添加先执行
   */


  _createClass(EventDispatcher, [{
    key: 'addEventListener',
    value: function addEventListener(type, fun) {
      var priority = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      if (!this.events.has(type)) this.events.set(type, []);

      //如果已添加过，不再添加
      if (this._searchFunIndexWithType(fun, type) >= 0) return;

      var newItem = { 'priority': priority, 'fun': fun };

      //找到优选级小于当前priority的位置插入，如果没有，则插入到末尾
      var arr = this.events.get(type);
      for (var i = 0; i <= arr.length; i++) {
        if (i === arr.length) {
          arr.push(newItem);
          break;
        } else if (arr[i].priority < priority) {
          arr.splice(i, 0, newItem);
          break;
        }
      }
    }

    /**
     * 查找某个监听处理方法在队列中的序号
     * @param {function} fun 
     * @param {string} type 
     * 
     * @private
     */

  }, {
    key: '_searchFunIndexWithType',
    value: function _searchFunIndexWithType(fun, type) {
      if (this.events == null || !this.events.has(type)) return -1;

      var arr = this.events.get(type);
      for (var i = 0; i < arr.length; i++) {
        var element = arr[i];
        if (element.fun === fun) return i;
      }
      return -1;
    }

    /**
     * 判断是否注册了针对某个事件类型的监听
     * @param {string} type 事件类型
     */

  }, {
    key: 'hasEventListener',
    value: function hasEventListener(type) {
      return this.events != null && this.events.has(type) && this.events.get(type) != null && this.events.get(type).length > 0;
    }

    /**
     * 将事件分派到事件流中
     * @param {Event} event 要发出的事件
     */

  }, {
    key: 'dispatchEvent',
    value: function dispatchEvent(event) {
      if (this.hasEventListener(event.type)) {
        if (event.target === null) {
          event.target = this;
        }
        if (event.currentTarget === null) {
          event.currentTarget = this;
        }
        var arr = this.events.get(event.type);
        for (var i = 0; i < arr.length; i++) {
          var item = arr[i];
          if (item.fun) item.fun(event);
        }
      }
    }

    /**
     * 移除事件监听
     * @param {string} type 事件类型
     * @param {function} fun 函数
     */

  }, {
    key: 'removeEventListener',
    value: function removeEventListener(type, fun) {
      if (this.hasEventListener(type)) {
        var index = this._searchFunIndexWithType(fun, type);
        if (index >= 0) this.events.get(type).splice(index, 1);
      }
    }
  }]);

  return EventDispatcher;
}();

exports.default = EventDispatcher;
//# sourceMappingURL=EventDispatcher.js.map