'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * 树结点数据结构
 * @author: xujunjie 
 * @Date: 2017-09-28 18:42:33 
 * @Last Modified by: xujunjie
 * @Last Modified time: 2017-11-22 11:55:49
 */
var TreeNodeData = function () {
	_createClass(TreeNodeData, null, [{
		key: 'createDataByArray',


		/**
   * 把指定Array转换成TreeNodeData的数组，默认格式如下
   * [ {value:'a'}, {value:'b'}, {value:'c', children:[ {value:1, value:2} ]} ]
   * 如果非默认格式，可通过参数指定每一项的值名称和子项列表名称
   * 
   * @param {*} arr 要转换的数组
   * @param {*} valueName 值名称，不指定则使用'value'
   * @param {*} childrenName 子项列表名称，不指定则使用'children'
   */
		value: function createDataByArray(arr) {
			var valueName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
			var childrenName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

			if (arr && arr.length > 0) {
				var VALUE_NAME = valueName ? valueName : 'value';
				var CHILDREN_NAME = childrenName ? childrenName : 'children';
				var result = [];
				for (var i = 0; i < arr.length; i++) {
					var item = new TreeNodeData(arr[i][VALUE_NAME]);
					result.push(item);
					//如果有子项，则创建子项
					if (arr[i][CHILDREN_NAME] && arr[i][CHILDREN_NAME].length > 0) {
						item.children = TreeNodeData.createDataByArray(arr[i][CHILDREN_NAME], valueName, childrenName);
					}
				}
				return result;
			}
			return null;
		}
	}]);

	function TreeNodeData(value) {
		var children = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
		var selected = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

		_classCallCheck(this, TreeNodeData);

		this._value = value;
		this._selected = selected;
		this._levelIndex = 0;
		this.children = children;
	}

	_createClass(TreeNodeData, [{
		key: '_updateChildrenLevelIndex',
		value: function _updateChildrenLevelIndex() {
			if (this.children !== null) {
				this.children.forEach(function (item) {
					item.levelIndex = this._levelIndex + 1;
				}, this);
			}
		}
	}, {
		key: 'selected',
		get: function get() {
			return this._selected;
		},
		set: function set(value) {
			if (this.selected !== value) {
				this._selected = value;
			}
		}

		/**
   * 当前结点携带的数据
   * @type {any}
   */

	}, {
		key: 'value',
		get: function get() {
			return this._value;
		},
		set: function set(value) {
			if (this._value !== value) {
				this._value = value;
			}
		}

		/**
   * 子结点集合，其中每一项是TreeNodeData
   * @type {Array}
   * @default null
   *
   */

	}, {
		key: 'children',
		get: function get() {
			return this._children;
		},
		set: function set(value) {
			if (this._children !== value) {
				this._children = value;
				this._updateChildrenLevelIndex();
			}
		}

		/**
   * 当前结点的层级
   * @type {number}
   * 
   * @default 0
   */

	}, {
		key: 'levelIndex',
		get: function get() {
			return this._levelIndex;
		},
		set: function set(value) {
			if (this._levelIndex !== value) {
				this._levelIndex = value;
				this._updateChildrenLevelIndex();
			}
		}
	}]);

	return TreeNodeData;
}();

exports.default = TreeNodeData;
//# sourceMappingURL=TreeNodeData.js.map