"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _EventDispatcher2 = require("../core/EventDispatcher");

var _EventDispatcher3 = _interopRequireDefault(_EventDispatcher2);

var _ProgressEvent = require("../events/ProgressEvent");

var _ProgressEvent2 = _interopRequireDefault(_ProgressEvent);

var _Loader = require("./Loader");

var _Loader2 = _interopRequireDefault(_Loader);

var _DataEvent = require("../events/DataEvent");

var _DataEvent2 = _interopRequireDefault(_DataEvent);

var _ErrorEvent = require("../events/ErrorEvent");

var _ErrorEvent2 = _interopRequireDefault(_ErrorEvent);

var _RemoteEvent = require("../events/RemoteEvent");

var _RemoteEvent2 = _interopRequireDefault(_RemoteEvent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * 多文件加载器
 * @author : xujunjie (yinhunfeixue@163.com)
 * @date   : 2017-12-28 13:22:45
 */
var MultipleLoader = function (_EventDispatcher) {
	_inherits(MultipleLoader, _EventDispatcher);

	/**
  * 
  * @param {*} maxLoadingNumber 同时进行的最大下载数
  */
	function MultipleLoader() {
		var maxLoadingNumber = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 3;

		_classCallCheck(this, MultipleLoader);

		var _this = _possibleConstructorReturn(this, (MultipleLoader.__proto__ || Object.getPrototypeOf(MultipleLoader)).call(this));

		_this.loader_progressHandler = function (event) {
			_this._updateProgress();
		};

		_this.loader_completeHandler = function (event) {
			_this._updateProgress();
			_this._autoLoad();
			var loader = event.currentTarget;
			if (loader.status === _Loader2.default.COMPLETE || loader.status === _Loader2.default.ERROR && _this._remainCountMap.get(loader.urlRequest) <= 0) {
				//如果加载成功，或者加载错误，但是剩余次数为0;发出单个加载器完成的事件
				if (_this.hasEventListener(_DataEvent2.default.ITEM_COMPLETE)) {
					_this.dispatchEvent(new _DataEvent2.default(_DataEvent2.default.ITEM_COMPLETE, event.currentTarget));
				}
			}
		};

		_this._requestList = []; //请求列表，每一项是一个UrlRequest
		_this._loaderMap = new Map(); //加载器键值对,格式为[urlRequest] = Loader
		_this._remainCountMap = new Map(); //剩余下载次数键值对,格式为[urlRequest] = 3
		_this._percent = 0; //当前加载的进度
		_this._maxLoadingNumber = maxLoadingNumber; //最大同时下载的数量
		_this._status = _Loader2.default.UNSTART;
		_this._usedTime = 0; //已用时间，如果一个加载器被多轮使用（加载完成后，再次添加项目加载），此值表示已经加载完成的那些轮总耗时，不包含本轮未完成的时间

		_this._startTime = null; //本轮开始下载的时间

		_this._lastLoadeBytes = 0;
		_this._lastProgressTime = 0;
		return _this;
	}

	/**
  * 当前状态
  * @property {string} Loading.COMPLETE 已结束
  * @property {string} Loading.LOADING 加载中
  * @property {string} Loading.UNSTART	未加载
  */


	_createClass(MultipleLoader, [{
		key: "hasRequest",
		value: function hasRequest(urlRequest) {
			return this._requestList.indexOf(urlRequest) >= 0;
		}

		/**
   * 添加下载请求，如果请求（urlRequest实例）已存在，不执行操作
   * @param {*} urlRequest 
   * @param {*} maxLoadCount 加载失败的情况下，最多尝试下载的次数（包含第一次）。默认为3，表示如果第1次失败，会再尝试2次
   */

	}, {
		key: "addItem",
		value: function addItem(urlRequest) {
			var maxLoadCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 3;

			//如果请求已存在，不执行操作
			if (this.hasRequest(urlRequest)) {
				return;
			}
			//把请求和剩余次数添加到列表中
			this._requestList.push(urlRequest);
			this._remainCountMap.set(urlRequest, maxLoadCount);
			this._updateProgress();
			this._autoLoad();
		}

		/**
   * 移除正在加载的项，如果加载正在进行，会尝试中止
   * @param {*} urlRequest 
   */

	}, {
		key: "removeItem",
		value: function removeItem(urlRequest) {
			var index = this._requestList.indexOf(urlRequest);
			if (index >= 0) {
				this._requestList.splice(index, 1);
				this._remainCountMap.delete(urlRequest);
				var loader = this._loaderMap.get(urlRequest);
				if (loader) {
					loader.close();
				}
				this._loaderMap.delete(urlRequest);
				this._updateProgress();
				this._autoLoad();
			}
		}
	}, {
		key: "_autoLoad",


		/**
   * 自动执行加载
   * 寻找下一个可下载的进行下载，如果没有，则不执行操作
   */
		value: function _autoLoad() {
			//查找处于加载状态的loader，如果小于允许的最大数量，则查找可用(剩余下载次数大于0)的下载请求
			var loadingCount = 0;
			var loaderList = this._loaderMap.values();
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = loaderList[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var value = _step.value;

					if (value.status === _Loader2.default.LOADING) {
						loadingCount++;
					}
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}

			if (loadingCount >= this._maxLoadingNumber) {
				return;
			}

			//寻找等待下载的请求进行下载,并把剩余次数减1
			var request = this._getWaitRequest();

			if (request) {
				var remainCount = this._remainCountMap.get(request);
				this._remainCountMap.set(request, --remainCount);

				//如果此请求还未创建loader，则创建loader。之后进行下载
				/**
     * @type {Loader}
     */
				var loader = void 0;
				if (this._loaderMap.has(request)) {
					loader = this._loaderMap.get(request);
				} else {
					loader = new _Loader2.default();
					loader.addEventListener(_ProgressEvent2.default.PROGRESS, this.loader_progressHandler);
					loader.addEventListener(_DataEvent2.default.COMPLETE, this.loader_completeHandler);
					loader.addEventListener(_ErrorEvent2.default.ERROR, this.loader_completeHandler);
					this._loaderMap.set(request, loader);
				}
				//如果是空闲状态，变更为加载状态
				if (this._status === _Loader2.default.UNSTART || this._status === _Loader2.default.COMPLETE) {
					this._status = _Loader2.default.LOADING;
					this._startTime = Date.now();
				}
				loader.load(request);

				//继续尝试加载，直到正在加载的请求等于最大允许数或者没有等待加载的请求
				this._autoLoad();
			}
			//如果没有等待加载的请求，判断是否所有请求都加载结束，如果结束，发出加载完成事件
			else {
					var isComplete = true;
					var _iteratorNormalCompletion2 = true;
					var _didIteratorError2 = false;
					var _iteratorError2 = undefined;

					try {
						for (var _iterator2 = this._requestList[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
							var _request = _step2.value;

							var _loader = this._loaderMap.get(_request);
							var remainNumber = this._remainCountMap.get(_request);

							//如果还有剩余下载次数，但是又不是处于下载结束状态，则整体加载没结束
							if (remainNumber > 0 && _loader.status !== _Loader2.default.COMPLETE && _loader.status !== _Loader2.default.ERROR) {
								isComplete = false;
								break;
							}
						}
					} catch (err) {
						_didIteratorError2 = true;
						_iteratorError2 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion2 && _iterator2.return) {
								_iterator2.return();
							}
						} finally {
							if (_didIteratorError2) {
								throw _iteratorError2;
							}
						}
					}

					if (isComplete) {
						this._status = _Loader2.default.COMPLETE;
						this._usedTime += Date.now() - this._startTime;
						this._startTime = null;
						if (this.hasEventListener(_RemoteEvent2.default.COMPLETE)) {
							this.dispatchEvent(new _RemoteEvent2.default(_RemoteEvent2.default.COMPLETE));
						}
					}
				}
		}
	}, {
		key: "_getWaitRequest",


		/**
   * 获取等待加载的请求
   * 剩余次数大于0，且对应的Loader不在加载状态
   */
		value: function _getWaitRequest() {
			for (var i = 0; i < this._requestList.length; i++) {
				var request = this._requestList[i];
				var loader = this._loaderMap.get(request);
				var remainCount = this._remainCountMap.get(request);
				if (remainCount > 0 && (!loader || loader.status === _Loader2.default.UNSTART || loader.status === _Loader2.default.ERROR)) {
					return request;
				}
			}
			return null;
		}
	}, {
		key: "_updateProgress",
		value: function _updateProgress() {
			//请求的数量为n，则总进度为n，有一个请求加载完成，进度增加1/n，对于正是进行的加载，进行为1/n * 此请求本身的进度
			var loadedPercent = 0;
			var maxNumber = this._requestList.length;
			for (var i = 0; i < maxNumber; i++) {
				var request = this._requestList[i];
				var loader = this._loaderMap.get(request);
				if (loader) {
					if (loader.status === _Loader2.default.COMPLETE || loader.status === _Loader2.default.ERROR) {
						loadedPercent += 1 / maxNumber;
					} else if (loader.status === _Loader2.default.LOADING && loader.byteTotal > 0) {
						loadedPercent += loader.byteLoaded / loader.byteTotal * 1 / maxNumber;
					}
				}
			}

			//如果当前进度和上次的进度不同，发出进度改变事件
			if (loadedPercent !== this._percent) {
				this._percent = loadedPercent;
				if (this.hasEventListener(_ProgressEvent2.default.PROGRESS)) {
					this.dispatchEvent(new _ProgressEvent2.default(_ProgressEvent2.default.PROGRESS, this._percent, 1));
				}
			}

			//数据一秒更新一次，为了更准砍反映即时速度，否则跳跃太大
			if (Date.now() - this._lastProgressTime > 1000) {
				this._lastLoadeBytes = this.loadedBytes;
				this._lastProgressTime = Date.now();
			}
		}
	}, {
		key: "status",
		get: function get() {
			return this._status;
		}

		/**
   * 当前下载进度
   */

	}, {
		key: "percent",
		get: function get() {
			return this._percent;
		}
	}, {
		key: "sucessedNumber",
		get: function get() {
			var loaderList = this._loaderMap.values();
			var result = 0;
			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = loaderList[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var loader = _step3.value;

					if (loader.status === _Loader2.default.COMPLETE) {
						result++;
					}
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			return result;
		}
	}, {
		key: "errorNumber",
		get: function get() {
			var loaderList = this._loaderMap.values();
			var result = 0;
			var _iteratorNormalCompletion4 = true;
			var _didIteratorError4 = false;
			var _iteratorError4 = undefined;

			try {
				for (var _iterator4 = loaderList[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
					var loader = _step4.value;

					if (loader.status === _Loader2.default.ERROR) {
						result++;
					}
				}
			} catch (err) {
				_didIteratorError4 = true;
				_iteratorError4 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion4 && _iterator4.return) {
						_iterator4.return();
					}
				} finally {
					if (_didIteratorError4) {
						throw _iteratorError4;
					}
				}
			}

			return result;
		}
	}, {
		key: "loadedBytes",
		get: function get() {
			var loaderList = this._loaderMap.values();
			var result = 0;
			var _iteratorNormalCompletion5 = true;
			var _didIteratorError5 = false;
			var _iteratorError5 = undefined;

			try {
				for (var _iterator5 = loaderList[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
					var item = _step5.value;

					if (item.status === _Loader2.default.COMPLETE || item.status === _Loader2.default.LOADING) {
						result += item.byteLoaded;
					}
				}
			} catch (err) {
				_didIteratorError5 = true;
				_iteratorError5 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion5 && _iterator5.return) {
						_iterator5.return();
					}
				} finally {
					if (_didIteratorError5) {
						throw _iteratorError5;
					}
				}
			}

			return result;
		}

		/**
   * 平均速度  字节/秒
   */

	}, {
		key: "avgSpeed",
		get: function get() {
			var loadedBytes = this.loadedBytes;
			var usedTime = this._usedTime;

			if (this._status === _Loader2.default.LOADING) {
				usedTime += Date.now() - this._startTime;
			}
			return loadedBytes * 1000 / usedTime;
		}

		/**
   * 即时速度  字节/秒
   */

	}, {
		key: "speed",
		get: function get() {
			if (this._status === _Loader2.default.LOADING) {
				return (this.loadedBytes - this._lastLoadeBytes) * 1000 / (Date.now() - this._lastProgressTime);
			}
			return 0;
		}
	}]);

	return MultipleLoader;
}(_EventDispatcher3.default);

exports.default = MultipleLoader;
//# sourceMappingURL=MultipleLoader.js.map