'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Point = require('../core/Point');

var _Point2 = _interopRequireDefault(_Point);

var _Rectangle = require('../core/Rectangle');

var _Rectangle2 = _interopRequireDefault(_Rectangle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * 
 * 
 * @author : xujunjie 
 * @mail   : yinhunfeixue@163.com
 * @date   : 2017/9/11 上午9:22:52
 */
var HtmlUtil = function () {
	function HtmlUtil() {
		_classCallCheck(this, HtmlUtil);
	}

	_createClass(HtmlUtil, null, [{
		key: 'getPositionToPage',


		/**
   * 获取元素相对页面左上角的坐标
   * @param {htmlElement} element HTML元素
   * 
   * @return {Point} 元素相对页面左上解的坐标，如果元素为Null,返回Null
   */
		value: function getPositionToPage(element) {
			if (element === null) {
				return null;
			}
			var marginLeft = parseFloat(element.style.marginLeft) || parseFloat(element.style.margin);
			var marginTop = parseFloat(element.marginTop) || parseFloat(element.style.margin);
			if (isNaN(marginLeft)) marginLeft = 0;
			if (isNaN(marginTop)) marginTop = 0;
			var result = new _Point2.default(element.offsetLeft - marginLeft, element.offsetTop - marginTop);
			if (element.offsetParent !== null) {
				result.addPoint(HtmlUtil.getPositionToPage(element.offsetParent));
			}
			return result;
		}

		/**
   * 获取元素相对页面左上角的显示区域
   * @param {*} element 
   * @return {Rectangle} 矩形区域
   */

	}, {
		key: 'getRectangleToPage',
		value: function getRectangleToPage(element) {
			var point = HtmlUtil.getPositionToPage(element);
			return new _Rectangle2.default(point.x, point.y, element.clientWidth, element.clientHeight);
		}

		/**
   * 获取元素style.left, style.top的数字值
   * @param {*} element 
   * @return {Point} 坐标点
   */

	}, {
		key: 'getLeftTop',
		value: function getLeftTop(element) {
			if (element === null) {
				return null;
			}
			var style = window.getComputedStyle(element);
			var x = parseFloat(style.left);
			var y = parseFloat(style.top);
			if (isNaN(x)) {
				x = 0;
			}
			if (isNaN(y)) {
				y = 0;
			}
			return new _Point2.default(x, y);
		}

		/**
   * 相对页面左上角的全局坐标转换为相对某元素的坐标
   * @param {Point} globalPosition 全局坐标
   * @param {HtmlElement} element 元素
   * 
   * @return {Point} globalPosition相对elements的坐标值，如果elements或globalPosition为null，返回globalPosition本身
   */

	}, {
		key: 'globalToLocal',
		value: function globalToLocal(globalPosition, element) {
			if (globalPosition === null || element === null) {
				return globalPosition;
			}

			var result = globalPosition.clone();
			var elementsOffset = HtmlUtil.getPositionToPage(element);
			return result.subPoint(elementsOffset);
		}

		/**
   * 获取页面滚动的坐标
   * @return {Point} 坐标点
   */

	}, {
		key: 'getWindowScrollPositon',
		value: function getWindowScrollPositon() {
			return new _Point2.default(document.body.scrollLeft || document.documentElement.scrollLeft, document.body.scrollTop || document.documentElement.scrollTop);
		}
	}, {
		key: 'setWindowScrollPosition',
		value: function setWindowScrollPosition(x, y) {
			if (document.body.scrollLeft) {
				document.body.scrollLeft = x;
			} else {
				document.documentElement.scrollLeft = x;
			}

			if (document.body.scrollTop) {
				document.body.scrollTop = y;
			} else {
				document.documentElement.scrollTop = y;
			}
		}

		/**
   * 获取页面尺寸
   * @return {Rectangle} 页面尺寸
   */

	}, {
		key: 'getWindowRectangle',
		value: function getWindowRectangle() {
			return new _Rectangle2.default(0, 0, document.body.clientWidth || document.documentElement.clientWidth, document.body.clientHeight || document.documentElement.clientHeight);
		}

		/**
   * 获取指定元素相对于浏览器窗口左上角的位置
   * @param {*} element 
   * 
   * @return {Point} 坐标点
   */

	}, {
		key: 'getPositionToWindow',
		value: function getPositionToWindow(element) {
			return HtmlUtil.getPositionToPage(element).subPoint(HtmlUtil.getWindowScrollPositon());
		}

		/**
   * 获取元素相对浏览器窗口左上角的显示区域
   * @param {*} element 
   * 
   * @return {Rectangle} 矩形区域
   */

	}, {
		key: 'getRectangleToWindow',
		value: function getRectangleToWindow(element) {
			var result = HtmlUtil.getRectangleToPage(element);
			var scroll = HtmlUtil.getWindowScrollPositon();
			result.x -= scroll.x;
			result.y -= scroll.y;
			return result;
		}

		/**
   * 获取鼠标事件的全局坐标，支持移动端事件
   * @param {Event} event 鼠标事件
   * @return {Point} 坐标点
   */

	}, {
		key: 'getMouseEventPosition',
		value: function getMouseEventPosition(event) {
			if (event.pageX) {
				return new _Point2.default(event.pageX, event.pageY);
			} else if (event.changedTouches && event.changedTouches[0]) {
				return new _Point2.default(event.changedTouches[0].pageX, event.changedTouches[0].pageY);
			}
			return null;
		}

		/**
   * 创建元素
   * @param {string} tag 元素类型
   * @param {string} className 样式名
   * @param {string} innerHTML 内容
   * 
   * @return {HTMLElement}
   */

	}, {
		key: 'createElement',
		value: function createElement(tag) {
			var className = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
			var innerHTML = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

			var result = document.createElement(tag);
			result.className = className ? className : "";
			result.innerHTML = innerHTML;
			return result;
		}
	}, {
		key: 'createHTMLEvent',
		value: function createHTMLEvent(type) {
			var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
			var bubbles = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
			var cancelAble = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

			var result = document.createEvent("HTMLEvents");
			result.data = data;
			result.initEvent(type, bubbles, cancelAble);
			return result;
		}
	}]);

	return HtmlUtil;
}();

exports.default = HtmlUtil;
//# sourceMappingURL=HtmlUtil.js.map