"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * ### 定时器类，具有以下功能
 * + 设置间隔时间、触发的总次数，并且可在运行时修改。
 * + 启动，暂停，重置定时器
 * + 添加/删除 定时器触发时的响应函数
 * + 添加/删除 定时器正常结束时的响应函数（例如设置了总触发5次，当第5次触发后，会触发响应函数）
 * + 可获取如下属性
 *  + interval--间隔时间
 *  + isRuning--是否在运行中
 *  + count--可触发的总次数
 *  + number--已触发的次数
 */
var Timer = function () {
  _createClass(Timer, [{
    key: "number",


    /**
     * 计时器已触发的总次数，从1开始计数
     */
    get: function get() {
      return this._number;
    }

    /**
     * 是否在运行中
     */

  }, {
    key: "running",
    get: function get() {
      return this._running;
    }

    /**
     * 触发时间间隔（毫秒）
     */

  }, {
    key: "interval",
    get: function get() {
      return this._interval;
    },
    set: function set(value) {
      if (this._interval !== value) {
        this._interval = value;
        if (this._running) {
          this._initTimer();
        }
      }
    }

    /**
     * 允许触发的总次数
     * 定时器运行中会立刻生效，非运行中在下次运行时生效
     */

  }, {
    key: "repeatCount",
    get: function get() {
      return this._repeatCount;
    },
    set: function set(value) {
      if (this._repeatCount !== value) {
        this._repeatCount = value;
      }
    }

    /**
     * 创建Timer实例
     * @param {*} interval 触发间隔（毫秒）
     * @param {*} count 触发总次数，默认为0，表示无限制 
     */

  }]);

  function Timer() {
    var interval = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 100;
    var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    _classCallCheck(this, Timer);

    //间隔
    this._interval = interval;
    //是否运行中
    this._running = false;
    //当前触发的次数
    this._number = 0;
    //可触发的总次数
    this._repeatCount = count;

    //计时器Id
    this._timerId = null;

    //定时器触发时要响应的函数列表
    this._arrTimerHandler = [];
    this._arrCompleteHandler = [];

    this._timerHandler = this._timerHandler.bind(this);
  }

  /**
   * 开始计时
   */


  _createClass(Timer, [{
    key: "start",
    value: function start() {
      if (!this._running) {
        this._initTimer();
        this._running = true;
      }
    }

    /**
     * 停止计时
     */

  }, {
    key: "stop",
    value: function stop() {
      this._clearTimer();
      this._running = false;
    }

    /**
     * 重置计时器
     */

  }, {
    key: "reset",
    value: function reset() {
      this.stop();
      this._number = 0;
    }

    /**
     * @private
     */

  }, {
    key: "_clearTimer",
    value: function _clearTimer() {
      if (this._timerId) {
        clearInterval(this._timerId);
      }
    }

    /**
     * @private
     */

  }, {
    key: "_initTimer",
    value: function _initTimer() {
      this._clearTimer();
      this._timerId = setInterval(this._timerHandler, this._interval);
    }

    /**
     * @private
     */

  }, {
    key: "_timerHandler",
    value: function _timerHandler() {
      //如果没有次数限制，或者当前次数小于允许的总次数，当前次数+1，执行队列中的所有函数
      if (this._repeatCount <= 0 || this._number < this._repeatCount) {
        this._number++;
        for (var i = 0; i < this._arrTimerHandler.length; i++) {
          this._arrTimerHandler[i](this._number);
        }
      }

      //如果有次数限制，且当前次数大于等于总次数，则清除定时器并执行完成队列中的函数
      if (this._repeatCount > 0 && this._number >= this._repeatCount) {
        this.stop();
        for (var _i = 0; _i < this._arrCompleteHandler.length; _i++) {
          this._arrCompleteHandler[_i]();
        }
      }
    }

    /**
     * 添加定时器触发的响应函数，一个函数只能添加一次。格式为fun(number)，
     * + number——当前触发的次数，例如4表示第4次触发
     * @param {*} timerHandler 响应函数
     */

  }, {
    key: "addTimerHandler",
    value: function addTimerHandler(handler) {
      if (this._arrTimerHandler.indexOf(handler) < 0) {
        this._arrTimerHandler.push(handler);
      }
    }

    /**
     * 移除定时器触发的响应函数
     * @param {*} handler 响应函数
     */

  }, {
    key: "removeTimerHandler",
    value: function removeTimerHandler(handler) {
      var index = this._arrTimerHandler.indexOf(handler);
      if (index >= 0) {
        this._arrTimerHandler.splice(index, 1);
      }
    }

    /**
     * 是否已添加指定的监听函数
     * @param {*} handler 
     * 
     * @return {boolean} 是否已添加
     */

  }, {
    key: "hasTimerHandler",
    value: function hasTimerHandler(handler) {
      return this._arrTimerHandler.indexOf(handler) >= 0;
    }

    /**
     * 添加执行完成的响应函数
     * @param {*} handler 
     */

  }, {
    key: "addCompleteHandler",
    value: function addCompleteHandler(handler) {
      if (this._arrCompleteHandler.indexOf(handler) < 0) {
        this._arrCompleteHandler.push(handler);
      }
    }

    /**
     * 移除执行完成的响应函数
     * @param {*} handler 
     */

  }, {
    key: "removeCompleteHandler",
    value: function removeCompleteHandler(handler) {
      var index = this._arrCompleteHandler.indexOf(handler);
      if (index >= 0) {
        this._arrCompleteHandler.splice(index, 1);
      }
    }
  }]);

  return Timer;
}();

exports.default = Timer;
//# sourceMappingURL=Timer.js.map